@@ -1,4 +1,4 @@
-//  $Id: rpsl_attr.cc,v 4.9.4.4 2003/07/24 11:21:00 katie Exp $
+//  $Id: rpsl_attr.cc,v 4.9.4.6 2003/10/17 08:38:30 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -117,34 +117,28 @@ ostream &AttrPeering::print(ostream &out) const {
    return out;
 }
 
-ostream &AttrMPPeering::print(ostream &out) const {
-   out << "peering: " << *peering;
-  // out << "********** NOT IMPLEMENTED ******* " << endl; //*peering;
-
-   return out;
-}
-
-
 ostream &AttrIfAddr::print(ostream &out) const {
-   static char buffer[128];
+/*   static char buffer[128];
    out << "ifaddr:\t" << int2quad(buffer, ifaddr.get_ipaddr())
        << " masklen " << ifaddr.get_length();
-   return out;
-}
-ostream &AttrInterface::print(ostream &out) const {
-   static char buffer[128];
-   out << "ifaddr:\t" ;
-   out << ifaddr;
+  return out;
+*/
+   out << "ifaddr/interface:\t" ;
+   out << ifaddr->get_ip_text();
    out << " masklen " << ifaddr->get_length();
+   if (action)
+     out << " action " << *action;
+   if (tunnel)
+     out << " tunnel " << *tunnel;
    return out;
 }
 
-
 ostream &AttrPeerOption::print(ostream &out) const {
    out << option << "(" << *args << ")";
    return out;
 }
 
+/*
 ostream &AttrPeer::print(ostream &out) const {
    out << "peer:\t" << protocol->name
        << " " << *peer << " ";
@@ -156,10 +150,11 @@ ostream &AttrPeer::print(ostream &out) const {
    }
    return out;
 }
+*/
 
-ostream &AttrMPPeer::print(ostream &out) const {
-   out << "mp-peer:\t" << protocol->name ;
-   //    << " " << *peer << " ";
+ostream &AttrPeer::print(ostream &out) const {
+   out << "peer/mp-peer:\t" << protocol->name
+       << " " << peer->get_ip_text() << " ";
    for (AttrPeerOption *nd = options->head(); nd; ) {
       nd->print(out);
       nd = options->next(nd);
@@ -169,7 +164,6 @@ ostream &AttrMPPeer::print(ostream &out) const {
    return out;
 }
 
-
 ostream &AttrTypedef::print(ostream &out) const {
    if (type)
       out << "typedef: " << name << " " << *type;   
@@ -450,3 +444,10 @@ ostream& AttrTRLabel::print(ostream &out) const {
    return out;
 }
 
+ostream& operator<<(ostream& stream, const Tunnel& p) {
+   stream << p.remote_ip->get_ip_text() << " ";
+   if (p.encapsulation)
+     stream << *p.encapsulation ;
+   return stream; 
+}
+
